#include "matrix.h"

/**************************************************************\
* Note: The arrays begin at index number 1 and NOT at index 0. *
\**************************************************************/

/*
** Functions for manipulation of vectors.
*/

DIANYSMAPTR Initialize_Dianysma (const int columns)
{
	DIANYSMAPTR new_vector = (DIANYSMAPTR) NULL;
	int			i;

	if (columns < 1)
		return	new_vector;		// Return NULL pointer to indicate error.

	new_vector = (DIANYSMAPTR) malloc (sizeof (DIANYSMA));
	new_vector->theseis = (int) columns;

	new_vector->sthlh = (double *) malloc (sizeof (double) * new_vector->theseis);
	for (i = 0; i < new_vector->theseis; i++)
		new_vector->sthlh [i] = (double) 0.0;

	return new_vector;
}

void	Destroy_Dianysma (DIANYSMAPTR vector)
{
	if (vector == (DIANYSMAPTR) NULL)
		return;

	free (vector->sthlh);
	free (vector);
}

int		Set_Dianysma (DIANYSMAPTR vector, const int position, const double number)
{
	if (vector == (DIANYSMAPTR) NULL)
		return	SET_NOT_INITIALIZED_VECTOR;

	if ((position > vector->theseis) || (position < 1))
		return	SET_DIANYSMA_OUT_OF_BOUNDS;

	vector->sthlh [position - 1] = (double) number;

	return	OK;
}

int		Get_Dianysma (DIANYSMAPTR vector, const int position, double * result)
{
	if (vector == (DIANYSMAPTR) NULL)
		return	GET_NOT_INITIALIZED_VECTOR;

	if (position > vector->theseis)
		return	GET_DIANYSMA_OUT_OF_BOUNDS;

	*result = (double) vector->sthlh [position - 1];

	return	OK;		// Everything was OK.
}

int		Swap_Columns_Dianysma (DIANYSMAPTR vector, const int column1, const int column2)
{
	double	temp1, temp2;

	if ( ((column1 < 1) || (column1 > vector->theseis)) || ((column2 < 1) || (column2 > vector->theseis)) )
		return SWAP_COLUMNS_DIANYSMA_OUT_OF_BOUNDS;

	Get_Dianysma (vector, column1, &temp1);
	Get_Dianysma (vector, column2, &temp2);
	Set_Dianysma (vector, column1, temp2);
	Set_Dianysma (vector, column2, temp1);

	return OK;
}

void	Pollaplasiasmos_Dianysmatwn (DIANYSMAPTR dianysma1, DIANYSMAPTR dianysma2, double * result)
{
	int		i;
	double	temp, temp2;

	/* O pinakas ginomeno prepei na exei ginei Initialized. */
	*result = (double) 0.0;
	for (i = 1; i <= dianysma1->theseis; i++) {
		Get_Dianysma (dianysma1, i, &temp);
		Get_Dianysma (dianysma2, i, &temp2);
		*result += temp * temp2;
	}
}


/*
** Functions for manipulation of arrays.
*/

PINAKASPTR	Initialize_Pinaka (const int rows, const int columns)
{
	PINAKASPTR	matrix = (PINAKASPTR) NULL;
	int			i;

	if ((rows < 1) || (columns < 1))
		return	matrix;		//Indicate error in bounds.

	matrix = (PINAKASPTR) malloc (sizeof (PINAKAS));
	matrix->size = (int) rows;
	matrix->grammh = (DIANYSMAPTR *) malloc (sizeof (DIANYSMAPTR) * matrix->size);

	for (i = 0; i < matrix->size; i++)
		matrix->grammh [i] = Initialize_Dianysma (columns);

	return	matrix;
}

void	Destroy_Pinaka (PINAKASPTR matrix)
{
	int		i;

	if (matrix == (PINAKASPTR) NULL)
		return;

	for (i = matrix->size - 1; i >= 0; i--)
		Destroy_Dianysma (matrix->grammh [i]);

	free (matrix->grammh);
	free (matrix);
}

int		Set_Pinaka (PINAKASPTR matrix, const int row, const int column, const double number)
{
	if (matrix == (PINAKASPTR) NULL)
		return	SET_NOT_INITIALIZED_MATRIX;

	if ((row < 1) || (row > matrix->size))
		return	SET_PINAKA_OUT_OF_BOUNDS;

	return	Set_Dianysma (matrix->grammh [row - 1], column, number);
}

int		Get_Pinaka (PINAKASPTR matrix, const int row, const int column, double * result)
{
	if (matrix == (PINAKASPTR) NULL)
		return	GET_NOT_INITIALIZED_MATRIX;

	if ((row < 1) || (row > matrix->size))
		return	GET_PINAKA_OUT_OF_BOUNDS;

	return	Get_Dianysma (matrix->grammh [row - 1], column, result);
}

int 	Swap_Lines (PINAKASPTR matrix, const int line1, const int line2)
{
	DIANYSMAPTR		temp;

	if ( ((line1 < 1) || (line1 > matrix->size)) || ((line2 < 1) || (line2 > matrix->size)) )
		return	SWAP_LINES_OUT_OF_BOUNDS;

	temp = matrix->grammh [line1 - 1];
	matrix->grammh [line1 - 1] = matrix->grammh [line2 - 1];
	matrix->grammh [line2 - 1] = temp;

	return	OK;
}

int		Swap_Columns_Pinaka (PINAKASPTR matrix, const int column1, const int column2)
{
	int		i;

	if ( ((column1 < 1) || (column1 > matrix->size)) || ((column2 < 1) || (column2 > matrix->size)) )
		return SWAP_COLUMNS_PINAKA_OUT_OF_BOUNDS;

	for (i = 0; i < matrix->size; i++)
		Swap_Columns_Dianysma (matrix->grammh [i], column1, column2);

	return OK;
}

void	Pollaplasiasmos_Pinakwn (PINAKASPTR pinakas1, PINAKASPTR pinakas2, PINAKASPTR ginomeno)
{
	int		i, j, r;
	double	result, temp, temp2;

	/* O pinakas ginomeno prepei na exei ginei Initialized. */
	for (i = 1; i <= pinakas1->size; i++) {
		for (j = 1; j <= pinakas2->grammh [0]->theseis; j++) {
			result = (double) 0.0;
			for (r = 1; r <= pinakas2->size; r++) {
				Get_Pinaka (pinakas1, i, r, &temp);
				Get_Pinaka (pinakas2, r, j, &temp2);
				result += temp * temp2;
			}
			Set_Pinaka (ginomeno, i, j, result);
		}
	}
}

void	Pollaplasiasmos_Pinaka_Dianysmatos (PINAKASPTR pinakas, DIANYSMAPTR dianysma, DIANYSMAPTR ginomeno)
{
	int		i, r;
	double	result, temp, temp2;

	/* O pinakas ginomeno prepei na exei ginei Initialized. */
	for (i = 1; i <= pinakas->size; i++) {
		result = (double) 0.0;
		for (r = 1; r <= pinakas->size; r++) {
			Get_Pinaka (pinakas, i, r, &temp);
			Get_Dianysma (dianysma, r, &temp2);
			result += temp * temp2;
		}
		Set_Dianysma (ginomeno, i, result);
	}
}

int		Get_Column (PINAKASPTR pinakas, const int column, DIANYSMAPTR pinakas_sthlh)
{
	double	temp;
	int		i;

	if ((pinakas == (PINAKASPTR) NULL) || (pinakas_sthlh == (DIANYSMAPTR) NULL))
		return	GET_COLUMN_PINAKA_UNDEFINED;

	if ((column < 1) || (column > pinakas->size))
		return	GET_COLUMN_PINAKA_OUT_OF_BOUNDS;

	if (pinakas->size != pinakas_sthlh->theseis)
		return	GET_COLUMN_PINAKA_INCOMPATIBLE_TYPES;

	for (i = 1; i <= pinakas->size; i++) {
		Get_Pinaka (pinakas, i, column, &temp);
		Set_Dianysma (pinakas_sthlh, i, temp);
	}

	return	OK;
}

int		Set_Column (PINAKASPTR pinakas, const int column, DIANYSMAPTR pinakas_sthlh)
{
	double	temp;
	int		i;

	if ((pinakas == (PINAKASPTR) NULL) || (pinakas_sthlh == (DIANYSMAPTR) NULL))
		return	SET_COLUMN_PINAKA_UNDEFINED;

	if ((column < 1) || (column > pinakas->size))
		return	SET_COLUMN_PINAKA_OUT_OF_BOUNDS;

	if (pinakas->size != pinakas_sthlh->theseis)
		return	SET_COLUMN_PINAKA_INCOMPATIBLE_TYPES;

	for (i = 1; i <= pinakas->size; i++) {
		Get_Dianysma (pinakas_sthlh, i, &temp);
		Set_Pinaka (pinakas, i, column, temp);
	}

	return	OK;
}

void	Typwse_Pinaka (PINAKASPTR matrix, char * onoma)
{
	int		rows = matrix->size, columns = matrix->grammh [0]->theseis;
	int		i, j;

	printf ("%s\n", onoma);
	for (i = 1; i <= (int) strlen (onoma); i++)
		printf ("-");
	printf ("\n");
	for (i = 1; i <= rows; i++) {
		for (j = 1; j <= columns; j++) {
			printf ("%09.4f  ", matrix->grammh [i-1]->sthlh [j-1]);
		}
		printf ("\n");
	}
	printf ("\n");
}
